@@ -515,6 +515,29 @@ class StagingBatch(models.Model):
             return False
     
     
+    def log(self, message, level='INFO', status=None, logger=None):
+        """Emit a log message at the specified log level and optionally update the ImportQueue object status attribute.
+        """
+        
+        if level not in ALLOWED_LOG_LEVELS:
+            raise Exception("StagingBatch item %s] Invalid log level specified: %s" % (self.pk, level))
+        else:
+            level = getattr(logging, level)
+        
+        if logger is None:
+            logger = default_logger
+        
+        if status is not None:
+            self.status = status
+        
+        self.message = message
+        self.save()
+        
+        logger_message = "[StagingBatch item %s] %s" % (self.pk, message)
+        logger.log(level, logger_message)
+        return True
+    
+    
     def get_absolute_url(self):
         return reverse('djhcup_staging.views.stagingbatch_detail', kwargs={'batch_id': self.pk})
     
@@ -528,12 +551,13 @@ class StagingQueue(models.Model):
     input_ht = '''
         In the case of staging operations with a single input, set
         this attribute equal to it. For staging operations with
-        multiple inputs, this is typically a reference to a CORE_SOURCE table.
+        multiple inputs, this is typically a reference to a CORE_SOURCE table,
+        but may also be a CHARGES_SOURCE table in some instances.
         '''
     stg_input = models.ForeignKey('StagingTable', related_name='stg_input',
                                   help_text=input_ht)
     charges_input = models.ForeignKey('StagingTable', related_name='charges_input',
-                                      help_text='CHARGES, if required.',
+                                      help_text='CHARGES, if required (*never* CHARGES_SOURCE).',
                                       null=True, default=None, blank=True)
     pr_input = models.ForeignKey('StagingTable', related_name='pr_input',
                                  help_text='PR, if required.',
@@ -699,7 +723,7 @@ class StagingQueue(models.Model):
             return False
     
     def __unicode__(self):
-        return "StagingQueue item %s [%s]: %s" % (self.pk, self.status, self.stg_input)
+        return "StagingQueue item %s [%s]" % (self.pk, self.status)
 
 
 class StagingTable(models.Model):
@@ -739,6 +763,21 @@ class StagingTable(models.Model):
     voided_at = models.DateTimeField(blank=True, null=True, default=None)
     #voided_by = models.ForeignKey(django user object) #unimplemented
     
+    # methods/properties for determining state and year
+    def _state_abbr(self):
+        if self.parent is None:
+            return self.imported_via.file.state.abbreviation
+        else:
+            return self.parent._state_abbr()
+    state = property(_state_abbr)
+        
+    def _year(self):
+        if self.parent is None:
+            return self.imported_via.file.year
+        else:
+            return self.parent.year
+    year = property(_year)
+    
     def __unicode__(self):
         return "StagingTable %d: %s" % (self.pk, self.name)
     
@@ -839,12 +878,12 @@ class Column(models.Model):
                 logger.info("Successfully matched Column %s with ReferenceColumn %s" % (self.pk, self.ref_col.pk))
                 return True
             elif len(qs) > 1:
-                logger.warning("Found %s ReferenceColumn entries matching Column %s; Column.integrity_status will be updated to UNAVAILABLE_AUTO. The entries were: %s." % (len(qs), self.pk, qs))
+                logger.warning("Found %s ReferenceColumn entries matching Column %s (%s); Column.integrity_status will be updated to UNAVAILABLE_AUTO. The entries were: %s." % (len(qs), self.pk, self.name, qs))
                 self.integrity_status = 'UNAVAILABLE_AUTO'
                 self.save()
                 return False
             else:
-                logger.info("Found no ReferenceColumn entry matching Column %s; Column.integrity_status will be updated to UNAVAILABLE_AUTO.")
+                logger.info("Found no ReferenceColumn entry matching Column %s (%s); Column.integrity_status will be updated to UNAVAILABLE_AUTO." % (self.pk, self.name))
                 self.integrity_status = 'UNAVAILABLE_AUTO'
                 self.save()
                 return False
