@@ -10,6 +10,7 @@ from copy import deepcopy
 
 # third party packages
 import pyhcup
+from pyhcup.db import pg_getcols
 from pyhcup.parser import df_wtl
 from pyhcup.parser import replace_sentinels
 from pyhcup.db import insert_sql as gen_insert_sql
@@ -337,7 +338,8 @@ def pg_import_file(self, importqueue_pk, drop_rawload=True):
             imported_via = load,
             name = stg_table_name,
             category = stcat,
-            records = affected_rows
+            records = affected_rows,
+            wip = False
         )
         
         imported.save()
@@ -712,7 +714,6 @@ def pg_stage(self, stagingqueue_pk):
         raise Exception(message)
             
     else:
-        
         stg_input = stg_q.stg_input
         input_tbl = stg_input.name
         now_hr = now.strftime("%Y%m%d%H%M%S")
@@ -889,6 +890,9 @@ def pg_stage(self, stagingqueue_pk):
             shoveled_message = 'Moved and pivoted wide core table %s to long PR table %s (%s rows)' % (input_tbl, output_tbl, affected_rows)
         
         elif category == 'CORE_PROCESSED':
+            flaggers = pyhcup.default_uflaggers()
+            uflag_names = [x.name for x in flaggers]
+            
             if in_src != 'CORE_SOURCE':
                 message = "Input StagingTable category must be CORE_SOURCE for CORE_PROCESSED staging. Got %s." % in_src
                 stg_q.fail(message, logger=logger)
@@ -896,37 +900,64 @@ def pg_stage(self, stagingqueue_pk):
             else:
                 tbl_core_source = input_tbl
                 stg_q.log('Input StagingTable CORE_SOURCE: %s' % tbl_core_source, logger=logger)
+                core_cols = map(lambda x: x.lower(), pg_getcols(cnxn, tbl_core_source))
             
-            if stg_q.uflag_input is None or stg_q.uflag_input.category != 'UFLAGS':
-                message = "Must provide uflag_input StagingTable with category UFLAGS for CORE_PROCESSED staging."
-                stg_q.fail(message, logger=logger)
-                raise Exception(message)
+            if stg_q.aha_input is not None:
+                if stg_q.aha_input.category != 'AHALINK_SOURCE':
+                    message = "If provided, aha_input must be StagingTable with category AHALINK_SOURCE for CORE_PROCESSED staging."
+                    stg_q.fail(message, logger=logger)
+                    raise Exception(message)
+                else:
+                    tbl_ahal_source = stg_q.aha_input.name
+                    stg_q.log('Input StagingTable AHALINK_SOURCE: %s' % tbl_ahal_source, logger=logger)
+                    ahal_cols = map(lambda x: x.lower(), pg_getcols(cnxn, tbl_ahal_source))
             else:
-                tbl_uflags = stg_q.uflag_input.name
-                stg_q.log('Input StagingTable UFLAGS: %s' % tbl_uflags, logger=logger)
+                tbl_ahal_source = None
             
-            if stg_q.aha_input is None or stg_q.aha_input.category != 'AHALINK_SOURCE':
-                message = "Must provide aha_input StagingTable with category AHALINK_SOURCE for CORE_PROCESSED staging."
-                stg_q.fail(message, logger=logger)
-                raise Exception(message)
+            if stg_q.uflag_input is not None:
+                if stg_q.uflag_input.category != 'UFLAGS':
+                    message = "If provided, uflag_input must be StagingTable with category UFLAGS for CORE_PROCESSED staging."
+                    stg_q.fail(message, logger=logger)
+                    raise Exception(message)
+                else:
+                    tbl_uflags = stg_q.uflag_input.name
+                    stg_q.log('Input StagingTable UFLAGS: %s' % tbl_uflags, logger=logger)
+                    uflag_cols = map(lambda x: x.lower(), pg_getcols(cnxn, tbl_uflags))
             else:
-                tbl_ahal_source = stg_q.aha_input.name
-                stg_q.log('Input StagingTable AHALINK_SOURCE: %s' % tbl_ahal_source, logger=logger)
+                tbl_uflags = None
             
-            # get a list of columns in the input tables so far
-            from pyhcup.db import pg_getcols
-            core_cols = map(lambda x: x.lower(), pg_getcols(cnxn, tbl_core_source))
-            uflag_cols = map(lambda x: x.lower(), pg_getcols(cnxn, tbl_uflags))
-            ahal_cols = map(lambda x: x.lower(), pg_getcols(cnxn, tbl_ahal_source))
+            if stg_q.dx_pr_grps_input is not None:
+                if stg_q.dx_pr_grps_input.category != 'DX_PR_GRPS_SOURCE':
+                    message = "If provided, dx_pr_grps_input must be StagingTable with category DX_PR_GRPS for CORE_PROCESSED staging."
+                    stg_q.fail(message, logger=logger)
+                    raise Exception(message)
+                else:
+                    tbl_dx_pr_grps = stg_q.dx_pr_grps_input.name
+                    stg_q.log('Input StagingTable DX_PR_GRPS: %s' % tbl_dx_pr_grps, logger=logger)
+                    dx_pr_grps_cols = map(lambda x: x.lower(), pg_getcols(cnxn, tbl_dx_pr_grps))
+            else:
+                tbl_dx_pr_grps = None
+            
+            if stg_q.severity_input is not None:
+                if stg_q.severity_input.category != 'SEVERITY_SOURCE':
+                    message = "If provided, severity_input must be StagingTable with category SEVERITY for CORE_PROCESSED staging."
+                    stg_q.fail(message, logger=logger)
+                    raise Exception(message)
+                else:
+                    tbl_severity = stg_q.severity_input.name
+                    stg_q.log('Input StagingTable SEVERITY: %s' % tbl_severity, logger=logger)
+                    severity_cols = map(lambda x: x.lower(), pg_getcols(cnxn, tbl_severity))            
+            else:
+                tbl_severity = None
             
             # scrub some extras, so we don't have duplicate columns
             omit_from_right = ['state', 'year', 'key'] + core_cols
-            ahal_cols = [x for x in ahal_cols if x.lower() not in omit_from_right]
             
             # check to see whether we need a separate dte table, based
             # on whether those columns are already in CORE_PROCESSED.
             if 'daystoevent' in core_cols and 'visitlink' in core_cols:
                 # we do not need a dte_source
+                # set it to None regardless of availability
                 tbl_dte_source = None
             else:
                 # we need a dte_source
@@ -953,33 +984,68 @@ def pg_stage(self, stagingqueue_pk):
                 select_items.append('tbl_dte_source.daystoevent AS daystoevent')
                 select_items.append('tbl_dte_source.visitlink AS visitlink')
                 jsql += """
-                LEFT JOIN {tbl_dte_source} AS tbl_dte_source
-                ON tbl_core_source.key = tbl_dte_source.key
-                """.format(tbl_dte_source=tbl_dte_source)
+                    LEFT JOIN {tbl_dte_source} AS tbl_dte_source
+                    ON tbl_core_source.key = tbl_dte_source.key
+                    """.format(tbl_dte_source=tbl_dte_source)
 
-            # join in the ahal_cols as well
+            # join in the ahalink cols
             if tbl_ahal_source is not None:
-
-                for a in ahal_cols:
+                ahal_sel_cols = [x for x in ahal_cols if x.lower() not in omit_from_right]
+                
+                for a in ahal_sel_cols:
                     select_items.append('tbl_ahal_source.{a} AS {a}'.format(a=a))
-                    
+                
+                #print ahal_cols
+                if 'key' in ahal_cols:
+                    ahal_join_field = 'key'
+                else:
+                    ahal_join_field = 'dshospid'
+                
                 jsql += """
-                LEFT JOIN {tbl_ahal_source} AS tbl_ahal_source
-                ON tbl_core_source.dshospid = tbl_ahal_source.dshospid 
-                """.format(tbl_ahal_source=tbl_ahal_source)
-
-            # specifically select in columns from tbl_uflags that match names
-            # in pyhcup.default_uflaggers(), and prepare them as JOIN clauses
-            flaggers = pyhcup.default_uflaggers()
-            uflag_names = [x.name for x in flaggers]
+                    LEFT JOIN {tbl_ahal_source} AS tbl_ahal_source
+                    ON tbl_core_source.{ahal_join_field} = tbl_ahal_source.{ahal_join_field}
+                    """.format(tbl_ahal_source=tbl_ahal_source, ahal_join_field=ahal_join_field)
             
-            for f in uflag_names:
-                select_items.append('tbl_{flag_name}.value AS {flag_name}'.format(flag_name=f))
+            # TODO: SEVERITY
+            if tbl_severity is not None:
+                # skip columns in omit_from_right (pk fields and CORE_SOURCE fields)
+                
+                for f in severity_cols:
+                    if f.lower() not in omit_from_right:
+                        select_items.append('tbl_severity.{f} AS {f}'.format(f=f))
+                        
                 jsql += """
-                LEFT JOIN {tbl_uflags} AS tbl_{flag_name}
-                ON tbl_{flag_name}.name = '{flag_name}' AND
-                    tbl_core_source.key = tbl_{flag_name}.key
-                """.format(tbl_uflags=tbl_uflags, flag_name=f, tbl_core_source=tbl_core_source)
+                    LEFT JOIN {tbl_severity} AS tbl_severity
+                    ON tbl_core_source.key = tbl_severity.key
+                    """.format(tbl_severity=tbl_severity)
+            
+            if tbl_dx_pr_grps is not None:
+                # skip columns in uflags
+                # select in columns from tbl_dx_pr_grps
+                # and prepare JOIN them
+                
+                for f in dx_pr_grps_cols:
+                    if f.upper() not in uflag_names and f.lower() not in omit_from_right:
+                        select_items.append('tbl_dx_pr_grps.{f} AS {f}'.format(f=f))
+                        
+                jsql += """
+                    LEFT JOIN {tbl_dx_pr_grps} AS tbl_dx_pr_grps
+                    ON tbl_core_source.key = tbl_dx_pr_grps.key
+                    """.format(tbl_dx_pr_grps=tbl_dx_pr_grps)
+            
+            if tbl_uflags is not None:
+                # specifically select in columns from tbl_uflags that match names
+                # in pyhcup.default_uflaggers(), and prepare them as JOIN clauses
+                
+                for f in uflag_names:
+                    select_items.append('tbl_{flag_name}.value AS {flag_name}'.format(flag_name=f))
+                    jsql += """
+                        LEFT JOIN {tbl_uflags} AS tbl_{flag_name}
+                        ON tbl_{flag_name}.name = '{flag_name}' AND
+                            tbl_core_source.key = tbl_{flag_name}.key
+                        """.format(tbl_uflags=tbl_uflags, flag_name=f)
+            
+            # TODO: DX_PR_GRPS
             
             # concat everything into a stonking huge SQL statement
             full_sql = 'CREATE TABLE {output_tbl} AS\nSELECT {selects}\nFROM {tbl_core_source} AS tbl_core_source {joins}' \
