@@ -8,6 +8,7 @@ import sys
 # third party packages
 import pandas as pd
 import pyhcup
+import datetime
 
 
 # django packages
@@ -16,6 +17,7 @@ from django.http import HttpResponse
 from django.utils.timezone import utc
 from django.conf import settings
 from django.core.urlresolvers import reverse
+from django.core.exceptions import ObjectDoesNotExist
 from django.db import connections as cnxns
 from django.db import transaction as tx
 from django.db.models import Q
@@ -23,8 +25,8 @@ from django.db.models import Q
 
 # local project and app packages
 import djhcup_staging
-from djhcup_staging.models import State, DataSource, FileType, File, Column, ImportQueue, ImportBatch, StagingTable, StagingBatch, ReferenceColumn
-from djhcup_staging.utils.misc import dead_batches
+from djhcup_staging.models import State, DataSource, FileType, File, Column, ImportQueue, ImportBatch, StagingTable, StagingBatch, StagingQueue, ReferenceColumn
+from djhcup_staging.utils import misc, staging
 
 
 # move these into the local namespace
@@ -336,6 +338,7 @@ def col_match(request):
     unmatch_cols = Column.objects.filter(ref_col=None)
     pass
 
+
 def src_gen_master_table(request, source_id):
     """For wide data, use all known loadfiles for source and generate one master table. For long data (or data which can be longed), make the corresponding fixed-style table.
     """
@@ -428,107 +431,22 @@ def src_gen_master_table(request, source_id):
 
 
 def create_batch_from_unimported(request):
-    now = datetime.datetime.utcnow().strftime("%Y%m%d%H%M%S")
-
-    name = 'Non-imported {now}' \
-        .format(now=now)
-
-    description = 'Non-imported files as of {now}' \
-        .format(now=now)
-
-    b = ImportBatch(name=name, description=description)
-    b.save()
-
-    fqs = File.objects.filter(
-        Q(file_type__content_type='CONTENTWH')|Q(file_type__content_type='CONTENTNH',loadfile__isnull=False)
-        ).exclude(state__abbreviation='TX') # these are special
-
-    fqs = fqs.order_by('state__abbreviation', 'year', '-file_type__compression')
-
-    # add each if one matching state-year is not already in there
-    for f in fqs:
-        # look for other matches that cover this already
-        st = f.state
-        yr = f.year
-        ds = f.file_type.source
-        
-        # either in the database
-        st_tbl_qs = StagingTable.objects.filter(
-            imported_via__file__state=st,
-            imported_via__file__year=yr,
-            imported_via__file__file_type__source=ds
-        )
-        
-        
-        # or in the queue set itself
-        q_qs = File.objects.filter(
-            state=st,
-            year=yr,
-            file_type__source=ds,
-            importqueue__batch=b
-        )
-        
-        if len(st_tbl_qs) == 0 and len(q_qs) == 0:
-            b.importqueue_set.create(file=f)
+    b = staging.unimported_batch()
     
-    message = 'Created ImportBatch {pk} with {length} files'.format(pk=b.pk, length=len(b.importqueue_set.all()))
+    message = 'Created ImportBatch {pk} with {length} files' \
+        .format(pk=b.pk, length=len(b.importqueue_set.all()))
     
     return importbatch_inventory(request, msg_pass_through=[message])
 
 
-def create_batch_from_unprocessed_t1(request):
-    messages = []
+def create_batch_from_unprocessed(request):
+    batches = staging.unprocessed_batches()
     
-    now = datetime.datetime.utcnow().strftime("%Y%m%d%H%M%S")
-
-    conversions = [
-        dict(cat_in='CHARGES_SOURCE', cat_out='CHARGES'),
-        dict(cat_in='CORE_SOURCE', cat_out='PR'),
-        dict(cat_in='CORE_SOURCE', cat_out='DX'),
+    messages = [
+        "[StagingBatch {pk} ({s})]: {n} ({l} items)" \
+        .format(pk=b.pk, s=b.status, n=b.name, l=b.stagingqueue_set.all())
+        for b in batches
     ]
-
-    for con in conversions:
-        cat_in = con['cat_in']
-        cat_out = con['cat_out']
-        name = '{cat_in} to {cat_out} {now}' \
-            .format(cat_in=cat_in, cat_out=cat_out, now=now)
-        description = 'Batch of all incomplete {cat_in} to {cat_out} StagingTables as of {now}.' \
-            .format(cat_in=cat_in, cat_out=cat_out, now=now)
-        b = StagingBatch(name=name, description=description)
-        b.save()
-        
-        src_qs = StagingTable.objects.filter(
-            category=con['cat_in'],
-            )
-
-        # add each if one matching state-year is not already in there
-        for t in src_qs:
-            # look for other matches that cover this already
-            f = t.imported_via.file
-            st = f.state
-            yr = f.year
-            
-            # either in the database
-            st_tbl_qs = StagingTable.objects.filter(
-                parent__imported_via__file__state=st,
-                parent__imported_via__file__year=yr,
-                category=con['cat_out'],
-            )
-            
-            # or in the queue set itself
-            q_qs = StagingQueue.objects.filter(
-                stg_input__parent__imported_via_file__state=st,
-                stg_input__parent__imported_via_file__year=yr,
-                category=con['cat_out']
-            )
-            
-            if len(st_tbl_qs) == 0 and len(q_qs) == 0:
-                b.stagingqueue_set.create(
-                    stg_input=t,
-                    category=con['cat_out'],
-                )
-    
-        messages.append('Created StagingBatch {pk} with {length} files'.format(pk=b.pk, length=len(b.stagingqueue_set.all())))
     
     return importbatch_inventory(request, msg_pass_through=messages)
 
@@ -541,11 +459,6 @@ def importbatch_inventory(request, msg_pass_through=[]):
     completed = batches.filter(complete__isnull=False)
     not_started = batches.filter(start__isnull=True)
     started = batches.filter(start__isnull=False, complete__isnull=True)
-    
-    dead = dead_batches()
-    if dead:
-        # don't include dead batches in the started listing
-        started = started.exclude(pk__in=[d.pk for d in dead])
 
     context = {
         'title': 'Batch inventory', 
@@ -555,7 +468,6 @@ def importbatch_inventory(request, msg_pass_through=[]):
             {'name': 'Started', 'value': started.count()},
             {'name': 'Completed', 'value': completed.count()},
             ],
-        'dead': dead,
         'not_started': not_started,
         'started': started,
         'completed': completed,
